using System;
using System.Collections;
using System.Data;
using System.Drawing;
using System.Text;

using CommonUI.Common.Collections; 

using Syncfusion.Grouping;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;

namespace CommonUI
{
	/// <summary>
	/// Summary description for GridDataBuilder.
	/// </summary>
	public class GridDataBuilder
	{

		public const string SUMMARY_NAME = "Total";// Is public because I need to remap custom summing

		private readonly GroupingGrid groupingGrid;
		private readonly GridColumnCollection displayColumnCollection;
		private readonly GridColumnCollection groupingColumnCollection;
		private readonly DataTable dataSource;
		private readonly GridColumnCollection sortByColumns;
		private readonly GridColumnCollection summaryColumnCollection;
		private readonly MergedColumnHeaderCollection mergedColumnHeaderCollection;

		private readonly double minEditValue;
		private readonly double maxEditValue;
		private readonly bool enableReadOnlyColumns;

		private ArrayList expandedGroups = null;
		private bool allGroupsExpanded;

		private readonly GridGroupingControl syncGrid; // this is set in the construcotr for ease reference

		public GridDataBuilder(
			GroupingGrid groupingGrid, 
			DataTable dataSource,
			GridColumnCollection displayColumnCollection,
			GridColumnCollection groupingColumnCollection,
			GridColumnCollection sortByColumns, 
			GridColumnCollection summaryColumnCollection,
			MergedColumnHeaderCollection mergedColumnHeaderCollection,
			double minEditValue, double maxEditValue, bool enableReadOnlyColumns) {
			//DBC.CheckPre("groupingGrid cannot be null", groupingGrid != null);
			//DBC.CheckPre("dataSource cannot be null", dataSource != null);
			//DBC.CheckPre("displayColumnCollection cannot be null", displayColumnCollection != null);
			//DBC.CheckPre("displayColumnCollection must have some items", displayColumnCollection.Count > 0);
			//DBC.CheckPre("groupingColumnCollection cannot be null", groupingColumnCollection != null);
			//DBC.CheckPre("sortByColumns cannot be null", sortByColumns != null);
			//DBC.CheckPre("summaryColumnCollection cannot be null", summaryColumnCollection != null);
			//DBC.CheckPre("mergedColumnHeaderCollection cannot be null", mergedColumnHeaderCollection != null);

			// Just make sure each mapping name appears in the datasource columns collection!
			ValidateParameters(dataSource, displayColumnCollection, groupingColumnCollection, 
				sortByColumns, summaryColumnCollection, mergedColumnHeaderCollection);

			this.dataSource = dataSource;
			this.groupingGrid = groupingGrid;
			this.displayColumnCollection = displayColumnCollection;
			this.groupingColumnCollection = groupingColumnCollection;
			this.sortByColumns = sortByColumns;
			this.summaryColumnCollection = summaryColumnCollection;
			this.mergedColumnHeaderCollection = mergedColumnHeaderCollection;
			this.minEditValue = minEditValue;
			this.maxEditValue = maxEditValue;
			this.enableReadOnlyColumns = enableReadOnlyColumns;

			// Set the readonly syncfusion grid
			this.syncGrid = groupingGrid.SyncfusionGridControl;
		}

		/// <summary>
		/// Performs the task of taking our gridcolum items and setting up the internal
		/// synfusion grid's collections.  This give us a layer of abstraction
		/// </summary>
		/// <param name="persistExpandedGroups">Remember any expanded groups</param>
		/// <param name="callEndUpdate">Pass in false if you've other stuff to do and will call endupdate yourself</param>
		public void Build(bool persistExpandedGroups) {
			// Just for my own sanity check for invoke required!!!!
			if (syncGrid.InvokeRequired) {
				throw new ApplicationException("syncGrid requires an Invoke but we're not set up for that!!");
			}
		
			if (persistExpandedGroups) {
				expandedGroups = new ArrayList();
				this.allGroupsExpanded = true;
				StoreExpandedGroups(syncGrid.Table.TopLevelGroup);
			}

			syncGrid.TableDescriptor.SummaryRows.Clear(); // It tihkn we must clear this now otherwise grid start to redraw and gets really confused and you gets loafds of catched errors from sync
			syncGrid.DataSource = dataSource;// Tie the datasource to the grid first		

			// JH Requires patch syncfusion grid v2.0.6.52
			// Wrap it in a try because i seems quite unreliable - particularly from unit tests
			try {
				if (syncGrid.TableModel.Table != null) {
					syncGrid.TableModel.Table.ResetRepaintElementsInQueue(); 
				}
			}
			catch (Exception ex) {
				////Logger.WriteLogEntry("Error in Build caught:" + ex.ToString());
			}

			// The syncfusiongrid auto populates lots of things when you've set the datasource
			ClearAutoPopulatedCollections();
			
			CreateDisplayColumns();
			CreateGroupingColumns();		
			//			HideGroupingColumns();
			AddSortByColumns();
			

			SetAllowScroll();

			// Add merged colums and the query covered range event handler
			// sorry but that's the only way you can get covered cells to work
			InitMergedColumnHeadersAndHandlers();

			if (persistExpandedGroups) {
				SetExpandedGroups();
			}

		}

		private void SetExpandedGroups() {
			if (expandedGroups.Count > 0
				&&
				this.groupingColumnCollection.Count > 0) {
				syncGrid.Table.TopLevelGroup.EnsureInitialized(syncGrid);
				if (this.allGroupsExpanded) {
					this.groupingGrid.ExpandAll();
				}
				else {
					for(int i = 0; i < expandedGroups.Count; i ++) {
						Group group = (Group)expandedGroups[i];
						ExpandPreviouslyExpandedGroups(syncGrid.Table.TopLevelGroup, group);
					}
				}
			}
		}

		private void ExpandPreviouslyExpandedGroups(Group parentGroup, Group itemToFind) {
			try {

				bool itemFound = false;
				if (parentGroup.Category == itemToFind.Category) {
					parentGroup.IsExpanded = true;
					itemFound = true;
				}

				if  (! itemFound) {
					int index = parentGroup.Groups.FindGroup(itemToFind.Category);
					if (index != -1) {
						parentGroup.Groups[index].IsExpanded = true;
					}
					else {
						for(int i = 0; i < parentGroup.Groups.Count; i ++) {
							Group group = parentGroup.Groups[i];
							ExpandPreviouslyExpandedGroups(group, itemToFind);
						}
					}
				}

				
			}
			catch (Exception ex) {
				////Logger.WriteLogEntry("Unreported msg " + ex.ToString());
			}
		}


		/// <summary>
		/// If there's grouping enabled we don't want auto scrolling to current row
		/// if that's on then expanding a cateogry will then get grid to jump to the current cell
		/// which could be miles down the gird
		/// </summary>
		private void SetAllowScroll() {
			GridScrollCurrentCellReason reason;

			if (this.groupingColumnCollection.Count == 0) {
				reason = GridScrollCurrentCellReason.Any;
			}
			else {
				reason = GridScrollCurrentCellReason.MoveTo;
			}

			syncGrid.TableModel.Options.AllowScrollCurrentCellInView = reason;
		}

		/// <summary>
		/// Clear any objects that were auto-populated by grid by setting datasource
		/// </summary>
		private void ClearAutoPopulatedCollections() {
			GridTableDescriptor tableDescriptor  = syncGrid.TableDescriptor;
			tableDescriptor.Columns.Clear();// 
			tableDescriptor.VisibleColumns.Clear();	
			tableDescriptor.GroupedColumns.Clear();	
			tableDescriptor.SummaryRows.Clear();	
			tableDescriptor.SortedColumns.Clear();
		}

		/// <summary>
		/// Tests to make sure GridColumn mapping names exist in the parameters passed in
		/// I should have changed Gridcolumn into a STRUCT to indicate that we're using
		/// values rather than references - ONE DAY...
		/// </summary>
		private void ValidateParameters(DataTable dataSource,
			GridColumnCollection displayColumnCollection,
			GridColumnCollection groupingColumnCollection,
			GridColumnCollection sortByColumns, 
			GridColumnCollection summaryColumnCollection,
			MergedColumnHeaderCollection mergedColumnHeaderCollection) {

			ValidateMappingNames(dataSource, displayColumnCollection);
			ValidateMappingNames(dataSource, groupingColumnCollection);
			ValidateMappingNames(dataSource, summaryColumnCollection);

			for(int i = 0; i < mergedColumnHeaderCollection.Count; i++) {
				MergedColumnHeader mergedColumnHeader = mergedColumnHeaderCollection[i];
				ValidateMappingName(dataSource, mergedColumnHeader.GridColumn1);
				ValidateMappingName(dataSource, mergedColumnHeader.GridColumn2);
			}
			
			ValidateMappingNames(dataSource, sortByColumns);

		}

		private void StoreExpandedGroups(Group parentGroup) {
			// Don't bother storing the TopLevel group - it has no keys! - and is always expanded anytway
			if (parentGroup is GridGroup) {
				if (parentGroup.IsExpanded) {
					expandedGroups.Add(parentGroup);
				}
				else {
					this.allGroupsExpanded = false;
				}
			}

			for(int i = 0; i < parentGroup.Groups.Count; i ++) {
				Group group = parentGroup.Groups[i];
				StoreExpandedGroups(group);
			}
		}

		private void ValidateMappingName(DataTable dataSource,	GridColumn gridColumn) {
			if ( ! dataSource.Columns.Contains(gridColumn.MappingName) ) {
				throw new ApplicationException("datasource does not contain a column called '" + gridColumn.MappingName + "'");				
			}
		}

		/// <summary>
		/// Makes sure that each mapping name does exist in the columns collection
		/// Delegates dwn to the single item
		/// </summary>
		private void ValidateMappingNames(DataTable dataSource,	ObjectSequence sequence) {
			// Can't do for(int i) because this[i] is protected
			foreach(GridColumn gridColumn in sequence) {
				ValidateMappingName(dataSource, gridColumn);
			}
		}


		/// <summary>
		/// Goes through the columns and adds them to the grid's
		/// </summary>
		private void CreateDisplayColumns() {

			// This is the editable stlye 
			GridTableCellStyleInfo editableStyle = new GridTableCellStyleInfo();
			editableStyle.BackColor = Color.White;
			editableStyle.ValidateValue.Minimum = this.minEditValue;
			editableStyle.ValidateValue.Maximum = this.maxEditValue;	
			editableStyle.Trimming = System.Drawing.StringTrimming.EllipsisCharacter;
			editableStyle.ReadOnly = false;

			// This is the read-only stlye
			GridTableCellStyleInfo readOnlyStyle = new GridTableCellStyleInfo();
			readOnlyStyle.BackColor = Color.LightCyan;	
			readOnlyStyle.CellType = "Static";
			readOnlyStyle.Trimming = System.Drawing.StringTrimming.EllipsisCharacter;
			readOnlyStyle.ReadOnly = true;
			readOnlyStyle.Enabled = this.enableReadOnlyColumns;

			//Combobox style editable
			GridTableCellStyleInfo comboBoxStyle = new GridTableCellStyleInfo();
			comboBoxStyle.BackColor = Color.White;
			comboBoxStyle.DropDownStyle = GridDropDownStyle.Exclusive;
			comboBoxStyle.CellType = "ComboBox";
			comboBoxStyle.Enabled = true;

			//Combobox style
			GridTableCellStyleInfo comboBoxStyleRO = new GridTableCellStyleInfo();
			comboBoxStyleRO.BackColor = Color.White;
			comboBoxStyleRO.CellType = "ComboBox";
			comboBoxStyleRO.ReadOnly = true;
			comboBoxStyleRO.Enabled = this.enableReadOnlyColumns;

			GridColumnDescriptorCollection columns = syncGrid.TableDescriptor.Columns;
			// JH put this extra check in as the cols shoud have been cleared out before we got here!!!
			if (columns.Count != 0) {
				throw new ApplicationException("columns should be 0 - failure to reset earlier");
			}

			GridVisibleColumnDescriptorCollection visibleColumns = syncGrid.TableDescriptor.VisibleColumns;
			// JH put this extra check in as the cols shoud have been cleared out before we got here!!!
			if (visibleColumns.Count != 0) {
				throw new ApplicationException("visibleColumns should be 0 - failure to reset earlier");
			}

			for(int i = 0; i < displayColumnCollection.Count; i ++) {
				GridColumn gridColumn = displayColumnCollection[i];

				// Note if you explcitly set the widht then it will not be auto recalced!
				GridColumnDescriptor colDescriptor = 
					new GridColumnDescriptor(gridColumn.MappingName, gridColumn.MappingName, gridColumn.Caption, gridColumn.IsReadOnly);

				// They've NOT asked for resize to fit then set the width
				if (! gridColumn.ResizeToFit) {
					colDescriptor.Width = gridColumn.Width;
				}

				if (gridColumn.IsReadOnly ) {
					if (gridColumn.UseComboBox) {
						colDescriptor.Appearance.AnyRecordFieldCell = comboBoxStyleRO;
						colDescriptor.Appearance.AnyRecordFieldCell.DataSource = gridColumn.DataSource;
						colDescriptor.Appearance.AnyRecordFieldCell.DisplayMember = gridColumn.DisplayMember;
						colDescriptor.Appearance.AnyRecordFieldCell.ValueMember = gridColumn.MappingName;
					}
					else {
						colDescriptor.Appearance.AnyRecordFieldCell = readOnlyStyle;
					}
				} 
				else {
					if (gridColumn.UseComboBox) {
						colDescriptor.Appearance.AnyRecordFieldCell = comboBoxStyle;
						colDescriptor.Appearance.AnyRecordFieldCell.DataSource = gridColumn.DataSource;
						colDescriptor.Appearance.AnyRecordFieldCell.DisplayMember = gridColumn.DisplayMember;
						colDescriptor.Appearance.AnyRecordFieldCell.ValueMember = gridColumn.MappingName;
					} 
					else {
						colDescriptor.Appearance.AnyRecordFieldCell = editableStyle;
					}
				}

				// Everyone gets the format string set - if there is one!
				if (gridColumn.FormatString != null) {
					colDescriptor.Appearance.AnyRecordFieldCell.Format = gridColumn.FormatString;
				}

				columns.Add(colDescriptor);

				// Add to the visible cols collection
				visibleColumns.Add(colDescriptor.Name);
			}
		}

		/// <summary>
		/// Goes through the grouping cols and adds them to the grid's
		/// group by list
		/// </summary>
		private void CreateGroupingColumns() {
			SortColumnDescriptorCollection sortColumns = syncGrid.TableDescriptor.GroupedColumns;

			for(int i = 0; i < groupingColumnCollection.Count; i ++) {
				GridColumn column = groupingColumnCollection[i];
				sortColumns.Add(column.MappingName);
			}
		}

		//		/// <summary>
		//		/// The display looks much better if you set the cols width to 0 for display cols that are grouped
		//		/// Thsi of course means there must be a dsiplay column for each  gropuing col!
		//		/// </summary>
		//		private void HideGroupingColumns()
		//		{
		//			GridColumnDescriptorCollection columns = syncGrid.TableDescriptor.Columns;
		//
		//			foreach(GridColumn gridColumn in groupingColumnCollection)
		//			{
		//				GridColumnDescriptor gridColumnDescriptor = columns[gridColumn.MappingName];
		//				if (gridColumnDescriptor != null)
		//				{
		//					//gridColumnDescriptor.Width = 0;
		//				}
		//			}
		//
		//		}

		/// <summary>
		/// Adds any of the sorting items to SortedColumns collection
		/// </summary>
		private void AddSortByColumns() {

			SortColumnDescriptorCollection sortCols = syncGrid.TableDescriptor.SortedColumns;

			for(int i = 0; i < this.sortByColumns.Count; i ++) {
				GridColumn gridColumn = this.sortByColumns[i];
				sortCols.Add(gridColumn.MappingName);
			}

		}


		/// <summary>
		/// Creates a bunch of GridSummaryColumnDescriptor items that are used to create
		/// summaries for specific cols
		/// </summary>
	
		/// <summary>
		/// This goes through and sets the HeaderText for the display columns
		/// and then adds groupingGrid.TableModel_QueryCoveredRange to the invocation list
		/// for syncGrid.TableModel.QueryCoveredRang
		/// </summary>
		private void InitMergedColumnHeadersAndHandlers() {	
			MergedColumnBuilder builder = new MergedColumnBuilder(mergedColumnHeaderCollection, groupingGrid);
			builder.Apply();
		}

		/// <summary>
		/// For debugging use
		/// </summary>
		public override string ToString() {
			StringBuilder builder = new StringBuilder();

			builder.AppendFormat("DataColumns:\n");
			foreach(DataColumn column in dataSource.Columns) {
				builder.Append("\t" + column.ColumnName + "\n");
			}

			builder.AppendFormat("DisplayColumns:\n");
			foreach(GridColumn gridCol in displayColumnCollection) {
				builder.Append("\t" + gridCol.Caption + "=" + gridCol.MappingName + ":W=" + gridCol.Width + "\n");
			}

			builder.AppendFormat("GroupingColumns:\n");
			foreach(GridColumn gridCol in groupingColumnCollection) {
				builder.Append("\t" + gridCol.Caption + "=" + gridCol.MappingName + ":W=" + gridCol.Width + "\n");
			}

			builder.AppendFormat("SortBy:\n");
			foreach(GridColumn gridCol in sortByColumns) {
				builder.Append("\t" + gridCol.Caption + "=" + gridCol.MappingName + ":W=" + gridCol.Width + "\n");
			}

			builder.AppendFormat("SummaryColumns:\n");
			foreach(GridColumn gridCol in summaryColumnCollection) {
				builder.Append("\t" + gridCol.Caption + "=" + gridCol.MappingName + "\n");
			}

			builder.AppendFormat("MergedColumns:\n");
			foreach(MergedColumnHeader mergedCol in mergedColumnHeaderCollection) {
				builder.Append("\t" + mergedCol.GridColumn1.MappingName + "&" + mergedCol.GridColumn2.MappingName + "=" + mergedCol.Caption + "\n");
			}

			return builder.ToString();
		}
	}
}
