using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Windows.Forms;

using Syncfusion.Drawing;
using Syncfusion.Grouping;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;

namespace CommonUI
{
	/// <summary>
	/// Summary description for GroupingGrid.
	/// </summary>
	public class GroupingGrid : System.Windows.Forms.UserControl
	{
		private GridGroupingControl grid;
		private Syncfusion.Grouping.GroupingConfig groupingConfig1;
		private System.ComponentModel.Container components = null;

		private MergedColumnHeaderCollection mergedColumnHeaderCollection = null;

		private GridStyleFormatter groupingStyle;		// Style object for Grouping look and feel
		private GridStyleFormatter headerStyle;			// Style for col headers
		private GridStyleFormatter dataRowStyle;		// Style for datarows - applies same style to odd and even

		private readonly Syncfusion.Windows.Forms.Grid.GridControlBase gridControlBase; 
		private readonly BrushInfo highlightRowBrush = new BrushInfo(SystemColors.Highlight);    // The colour of the highlited row - cache for speed      

		private CurrencyManager currencyManager = null;
		private System.Windows.Forms.PictureBox screenShot; //used for the find and row selection stuff - omly set when datasource is set
		//private Version3CurrentCellWorkaround version3CurrentCellWorkaround;
		private string primaryKeyColName = null;// Only allows 1 length col names!!!
		
		#region Constructor
		public GroupingGrid() {

			InitializeComponent();

			grid.SuspendLayout();

			InitializeBaseStyles();	// Init the styles that are publicly available

			grid.TableDescriptor.AllowCalculateMaxColumnWidth = true;
			
			grid.TableModel.CellModels.Add("CheckBoxEx", new GridCheckBoxCellModelEx(grid.TableModel));
			//grid.TableModel.SelectionChanged +=new Syncfusion.Windows.Forms.Grid.GridSelectionChangedEventHandler(TableModel_SelectionChanged);		
			//grid.TableControlMouseDown +=new GridTableControlMouseEventHandler(gridGroupingControl1_TableControlMouseDown);

			// Default to selecting all contents of a cell when selected
			grid.TableModel.Options.ActivateCurrentCellBehavior = GridCellActivateAction.SelectAll; 

			gridControlBase = grid.TableControl; // REAODNLY var

			//gridControlBase.QueryScrollCellInView += new GridQueryScrollCellInViewEventHandler(gridControlBase_QueryScrollCellInView);

			//CreateCurrentCellWorkaround();

			// See ProcessDialogKey - raised by JRM enter key loses focus
			grid.TableControl.WantEnterKey = false;
			//grid.TableControl.WantTabKey = false;// JH Don't set to false as the grid won;t scroll to the right 

			grid.ResumeLayout();		
		}
		#endregion

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Private Style methods

		/// <summary>
		/// Initialises our base styles for cetain aspects of the Grid
		/// </summary>
		private void InitializeBaseStyles() {

			// We want summary vales to be right aligned so set GroupCaptionSummaryCell directly
			grid.Appearance.GroupCaptionSummaryCell.HorizontalAlignment = GridHorizontalAlignment.Right;
			grid.Appearance.GroupIndentCell.BackColor = this.BackColor;	
			grid.Appearance.GroupCaptionSummaryCell.Clickable = false;// If it's clickable they keep losing the active row!!!

			grid.TableControl.BackColor = this.BackColor;

			// Just reference the appearance directly for speed
			GridTableCellAppearance appearance = grid.TableDescriptor.Appearance;


			// The grouping style is based on GroupCaptionCell, GroupCaptionSummaryCell
			//used to have 
			groupingStyle = new GridStyleFormatter( 
				new GridTableCellStyleInfo[] {
												 appearance.GroupCaptionCell, 
												 appearance.GroupCaptionSummaryCell});
			groupingStyle.ApplyStyle( GetDefaultGroupingStyle() );

			// The header style is based on just appearance.ColumnHeaderCell
			headerStyle = new GridStyleFormatter( 
				new GridTableCellStyleInfo[] {appearance.ColumnHeaderCell});
			headerStyle.ApplyStyle( GetDefaultHeaderStyle() );

			// We apply the same style to both odd and even rows - although you can have different
			// stlyes to alternate
			dataRowStyle = new GridStyleFormatter( new GridTableCellStyleInfo[] {
																					appearance.RecordFieldCell, appearance.AlternateRecordFieldCell});
			dataRowStyle.ApplyStyle( GetDefaultDataRowStyle());

		}

		public GridTableCellStyleInfo GetDefaultGroupingStyle() {
			GridTableCellStyleInfo newStyle = new GridTableCellStyleInfo();
			newStyle.BackColor = Color.Wheat;
			newStyle.Enabled = false;
			newStyle.ReadOnly = true;
			newStyle.Font.Bold = true;
			return newStyle;
		}

		public GridTableCellStyleInfo GetDefaultHeaderStyle() {
			GridTableCellStyleInfo newStyle = new GridTableCellStyleInfo();
			newStyle.CellAppearance = GridCellAppearance.Raised;
			newStyle.BackColor = SystemColors.Control;// Color..Silver;
			newStyle.Font.Bold = true;

			return newStyle;
		}

		public GridTableCellStyleInfo GetDefaultDataRowStyle() {
			GridTableCellStyleInfo newStyle = new GridTableCellStyleInfo();
			newStyle.BackColor = Color.LightCyan;
			newStyle.Font.Bold = false;
			
			return newStyle;
		}
		#endregion

		#region ProcessDialogKey
		/// <summary>
		/// Handles ENTER key being pressed
		/// </summary>
		/// <param name="keyData"></param>
		/// <returns></returns>
		protected override bool ProcessDialogKey(Keys keyData) {

			bool returnValue;

			if (keyData == Keys.Enter) {
				
				returnValue = true;

				try {
					GridCurrentCell cc = this.grid.TableControl.CurrentCell;
					if (cc != null) {
						cc = cc.Renderer.GetNestedCurrentCell();
						cc.ConfirmChanges();
						cc.Lock();
						((GridTableControl)cc.Grid).Table.CurrentRecordManager.EndEdit();
						cc.Unlock();		
						cc.Refresh();

						GridTextBoxCellRenderer tb = cc.Renderer as GridTextBoxCellRenderer;
						if (tb != null) {
							// Must set to 0th position
							tb.TextBox.SelectionStart = 0;
							tb.TextBox.SelectAll();
						}

						grid.Refresh();
					}
				
				}
				catch(Exception ex) {
					// There seems to be a null object sometimes - I think it might be 
					// when getting the currentcell but I cna;t seem to reproduce it
					////Logger.WriteLogEntry("Caught but not shown:" + ex.ToString());
					Console.WriteLine(ex.Message);
				}
			}
			else {
				returnValue = base.ProcessDialogKey(keyData);
			}
				
			return returnValue;

		}
		#endregion

		#region InitialiseWithGridDataBuilder
	
		public void InitialiseWithGridDataBuilder(
			DataTable dataSource, 
			GridColumnCollection displayColumnCollection, 
			GridColumnCollection groupingColumnCollection,
			GridColumnCollection sortByColumns,
			MergedColumnHeaderCollection mergedColumnHeaderCollection,
			GridColumnCollection summaryColumnCollection,
			double minEditValue, double maxEditValue, bool disableReadOnlyColumns) {
			//DBC.CheckPre("dataSource cannot be null", dataSource != null);
			//DBC.CheckPre("displayColumnCollection cannot be null", displayColumnCollection != null);
			//DBC.CheckPre("displayColumnCollection must have at least one item", displayColumnCollection.Count > 0);
			//DBC.CheckPre("groupingColumnCollection cannot be null", groupingColumnCollection != null);
			//DBC.CheckPre("sortByColumns cannot be null", sortByColumns != null);
			//DBC.CheckPre("mergedColumnHeaderCollection cannot be null", mergedColumnHeaderCollection != null);
			//DBC.CheckPre("summaryColumnCollection cannot be null", summaryColumnCollection != null);

			this.mergedColumnHeaderCollection = mergedColumnHeaderCollection; // Thsi needs to be used on the grid at runtime in TableModel_QueryCoveredRange

			GridDataBuilder builder = new GridDataBuilder(
				this, dataSource, 
				displayColumnCollection, groupingColumnCollection, 
				sortByColumns, summaryColumnCollection,
				mergedColumnHeaderCollection,
				minEditValue, maxEditValue, disableReadOnlyColumns);


			builder.Build(true);	// Remember expanded groups 
		}
		#endregion

		#region Public Methods

		/// <summary>
		/// Expands all groups
		/// </summary>
		public void ExpandAll() {
			grid.Table.ExpandAllGroups();
		}


		public void ClearSummaries() {
			grid.Table.TableDirty = true;  // Stops grid from recalcing summary objects 
			grid.Table.SummariesDirty = true;
		}

		public void BeginUpdate() {
			grid.TableModel.BeginUpdate();
		}


		/// <summary>
		/// Redraws grid after loads of changes
		/// </summary>
		public void EndUpdate(bool discardPendingPaints) {
			grid.TableModel.EndUpdate(discardPendingPaints);
		}

		/// <summary>
		/// Redraws grid after loads of changes
		/// </summary>
		public void EndUpdate() {
			grid.TableModel.EndUpdate();
		}

		/// <summary>
		/// Collapses all groups
		/// </summary>
		public void CollapseAll() {
			grid.Table.CollapseAllGroups();
		}

		/// <summary>
		/// If you're updating the grid directly from code and don;t want that to be logged
		/// in the undo stack call or events raised use this method - don't forget to clear though
		/// </summary>
		public void BeginDirectUpdateMode() {
			grid.TableModel.SuspendRecordUndo();
			grid.TableModel.SuspendChangeEvents();
		}

		/// <summary>
		/// Used in conjunction with BeginDirectUpdateMode
		/// </summary>
		public void EndDirectUpdateMode() {
			grid.TableModel.ResumeRecordUndo();
			grid.TableModel.ResumeChangeEvents();
		}

		/// <summary>
		/// Returns the Primary key of current record - note only supports a single primary key column
		/// </summary>
		/// <returns></returns>
		public object CurrentPrimaryKey() {
			object key = null;
			if (grid.Table != null
				&&
				grid.Table.CurrentRecord != null) {
				key = GetPrimaryKey(grid.Table.CurrentRecord);
			}

			return key;
		}

		/// <summary>
		/// Gets the primary key for the specific record - see CurrentPrimaryKey
		/// </summary>
		/// <param name="record"></param>
		/// <returns></returns>
		public object GetPrimaryKey(Record record) {
					
			object objectKey = null;

			try {
				// For some reason it seems to everynow and htne get a null ref
				objectKey = record.GetValue(primaryKeyColName);
			}
			catch (Exception ex) {
				////Logger.WriteLogEntry("Info: GetPrimaryKey erorr" + ex.ToString());
			}

			return objectKey;
		}

		public int RecordCount {
			get { return this.GridTableModel.Table.Records.Count;}
		}

		public void MoveLast() {
			this.GridTableModel.Table.UnsortedRecords[this.GridTableModel.Table.UnsortedRecords.Count - 1].SetCurrent();
			this.ScrollCurrentRecordIntoView();
		}

		public void OverlayCurrentScreenShot(bool applyGrayScale) {
			
		}

		public void HideCurrentScreenShot() {
			screenShot.Image = null;
			screenShot.SendToBack();
		}

		
		public void ScrollCurrentRecordIntoView() {
			Syncfusion.Grouping.Record currentRecord = this.GridTableModel.Table.CurrentRecord;
			if (currentRecord != null) {

				this.GridTableModel.Table.ShowRecord(currentRecord, true);

				bool scrolled = this.grid.TableControl.ScrollInView(currentRecord);

				// Some quirky behaviour again...it deosn;t always scroll even we we've asked it to!!!
				if (! scrolled) {
					// cache the current scroll options as we set them
					GridScrollCurrentCellReason currentReason = this.GridTableModel.Options.AllowScrollCurrentCellInView;
					this.GridTableModel.Options.AllowScrollCurrentCellInView = GridScrollCurrentCellReason.Any;
					try {
						scrolled = this.grid.TableControl.ScrollInView(currentRecord);
					}
					finally {
						this.GridTableModel.Options.AllowScrollCurrentCellInView = currentReason;
					}

#if (DEBUG)
					////Logger.WriteLogEntry("ScrollCurrentRecordIntoView : scrolled=" + scrolled);
#endif
				}

			}
		}

		#region IgnoreCancelTopRowChanging
		private bool ignoreTopRowChanging = false;

		/// <summary>
		/// There's an issue with the syncfusion gird
		/// http://www.syncfusion.com/Support/Forums/message.aspx?MessageID=29976
		/// If you try to set the toprow/col before calling EndUpdate it will reset
		/// row to 0 when you do call EndUpdate - this is an attempt to work around it
		/// JH 15/JUN/05 - it still doesnt seem towork though!!!!
		/// </summary>
		public bool IgnoreTopRowChanging {
			get { return ignoreTopRowChanging; }
			set {
				ignoreTopRowChanging = value;
				if (value) {
					grid.TableControlTopRowChanging += new GridTableControlRowColIndexChangingEventHandler(grid_TableControlTopRowChanging);
				}
				else {
					grid.TableControlTopRowChanging -= new GridTableControlRowColIndexChangingEventHandler(grid_TableControlTopRowChanging);
				}
			}
 
		}
		private void grid_TableControlTopRowChanging(object sender, GridTableControlRowColIndexChangingEventArgs e) {	
			e.Inner.Cancel = true;
		}
		#endregion

		public int GridTopRowIndex {
			get { return this.grid.TableControl.TopRowIndex; }
			set { this.grid.TableControl.TopRowIndex = value; }
		}

		public int GridLeftColIndex {
			get { return this.grid.TableControl.LeftColIndex; }
			set { this.grid.TableControl.LeftColIndex = value; }
		}

		public int VScrollPixelPos {
			get { return this.grid.TableControl.GetCurrentVScrollPixelPos(); }
			set { this.grid.TableControl.SetCurrentVScrollPixelPos(value); }
		}

		public int HScrollPixelPos {
			get { return this.grid.TableControl.GetCurrentHScrollPixelPos(); }
			set { this.grid.TableControl.SetCurrentHScrollPixelPos(value); }
		}

		public int VisibleRows {
			get { return this.grid.TableControl.ViewLayout.VisibleRows;}
		}

		public int VisibleCols {
			get { return this.grid.TableControl.ViewLayout.VisibleCols;}
		}

		/// <summary>
		/// Clears the selections but if you call this before endupdate you'll get
		/// an exception raised by the Syncfusion grid
		/// </summary>
		public void ClearSelections() {
			if (this.grid.TableControl.Selections.Count > 0) {
				this.grid.TableControl.Selections.Clear(); 
			}
		}

		/// <summary>
		/// Highlights the reocrd with the specified key
		/// </summary>
		public void SelectByKey(object primaryKeyValue) {
			// Work out hte index in the records
			int recordIndex = grid.Table.PrimaryKeySortedRecords.FindRecord(primaryKeyValue);
			if (recordIndex != -1) {
				Record foundRecord = grid.Table.PrimaryKeySortedRecords[recordIndex];

				// Now get the display index - this couldbe different to the record index
				// if they've got gripuing siwtched on or sorting
				int displayIndex = this.GridTableModel.Table.DisplayElements.IndexOf(foundRecord);
				if (displayIndex != -1) {
					GridTableModel.Table.ShowRecord(foundRecord, true);
					grid.TableControl.ScrollInView(foundRecord);

					GridControlBase.Selections.Clear(); 
					GridControlBase.Selections.Add(GridRangeInfo.Row(displayIndex));
				}
			}
		}

		/// <summary>
		/// Makes the record with specified Key current
		/// </summary>
		public void SetCurrentRecord(object primaryKeyValue) {
			
			
			try {		

				int index = grid.Table.PrimaryKeySortedRecords.FindRecord(primaryKeyValue);
				if (index != -1) {
					Record foundRecord = grid.Table.PrimaryKeySortedRecords[index];
					if (foundRecord != null) {
						foundRecord.SetCurrent();
					}
					//ScrollCurrentRecordIntoView();
				}
			}
			catch (Exception ex) {
				////Logger.WriteLogEntry(ex.ToString());
			}

		}

		#endregion

		#region Public Properties

		
		/// <summary>
		/// Gets or sets a value if editing records should be allowed when the the underlying source list supports editing records.  
		/// </summary>
		public bool AllowEdit {
			get { return this.grid.TableDescriptor.AllowEdit; }
			set { this.grid.TableDescriptor.AllowEdit = value; }
		}

		/// <summary>
		/// Gets or sets a value if adding new records should be allowed when the the underlying source list supports adding records. 
		/// </summary>
		public bool AllowNew {
			get { return this.grid.TableDescriptor.AllowNew; }
			set { this.grid.TableDescriptor.AllowNew = value; }
		}

		/// <summary>
		/// Gets or sets a value if removing records should be allowed when the the underlying source list supports removing records.  
		/// </summary>
		public bool AllowRemove {
			get { return this.grid.TableDescriptor.AllowRemove; }
			set { this.grid.TableDescriptor.AllowRemove = value; }
		}

		/// <summary>
		/// get's the collection of Syncfusion.Grouping.SortColumnDescriptorCollection GropuingColumns
		/// </summary>
		[Browsable(false)]
		public SortColumnDescriptorCollection GroupedColumns {
			get { return grid.TableDescriptor.GroupedColumns; }
		}

		/// <summary>
		/// Returns the currenctly active cell
		/// </summary>
		[Browsable(false)]
		public GridCurrentCell CurrentCell {
			get { return gridControlBase.CurrentCell; }
		}
		
		/// <summary>
		/// Gets the collection of Syncfusion.Grouping.RecordFilterDescriptor objects defining filter 
		/// criteria for records in the table. 
		/// Each Syncfusion.Grouping.RecordFilterDescriptor in the collection references one or 
		/// multiple Syncfusion.Grouping.FieldDescriptor of the 
		/// Syncfusion.Grouping.TableDescriptor.Fields collection. 
		/// Multiple Criteria can be combined with logical "And" or "Or" operations.  
		/// </summary>
		[Browsable(false)]
		public RecordFilterDescriptorCollection RecordFilters {
			get { return grid.TableDescriptor.RecordFilters; }
		}

		/// <summary>
		/// Set or gets the DataTable bound to the grid
		/// </summary>
		[Browsable(false)]
		public DataTable DataSource {
			get { return (DataTable)grid.DataSource; } // Note it's explicitly cast to a DataTable
			set { 
				grid.DataSource = value ;

				// we need to get into the binding context to get the highlight row stuff working
				if (value != null) {
					this.currencyManager = this.BindingContext[value] as CurrencyManager;
					this.currencyManager.PositionChanged += new EventHandler(cm_PositionChanged);
					this.primaryKeyColName = value.PrimaryKey[0].ColumnName;
				}

			}
		}

		/// <summary>
		/// Determines if the user can  drag cols around
		/// </summary>
		public bool AllowDragSelectedCols {
			get { return grid.TableModel.Options.AllowDragSelectedCols;}
			set { grid.TableModel.Options.AllowDragSelectedCols = value;}
		}

		/// <summary>
		/// Define the postion from far left of the frozen cols
		/// </summary>
		[Browsable(false)]
		public int FrozenColumns {
			get { return grid.TableModel.Cols.FrozenCount;}
			set { grid.TableModel.Cols.FrozenCount = value;}
		}

		//		public int RecordCount
		//		{
		//			get{ return grid.TableModel.Table.Records.Count;}
		//		}
		/// <summary>
		/// Defines the selection behaviour when a cell is selected
		/// None - Do not set focus to TextBox.  
		/// ClickOnCell - Begin editing / Focus on TextBox after user clicked on cell  
		/// SetCurrent - Begin editing / Focus on TextBox whenever a cell becomes current cell no matter if user clicked on cell or moved with arrow keys.  
		/// DblClickOnCell - Begin editing / Focus on TextBox when user double clicked on cell  
		/// SelectAll - Begin editing / Focus on TextBox and select all text whenever a cell becomes current cell no matter if user clicked on cell or moved with arrow keys.  
		/// PositionCaret - Forward mouse click to the TextBox so that the caret can be positioned at the character under the mouse pointer.  
		/// </summary>
		public GridCellActivateAction CellActivationBehaviour {
			get { return grid.TableModel.Options.ActivateCurrentCellBehavior; }
			set { grid.TableModel.Options.ActivateCurrentCellBehavior = value; }
		}

		private bool clearCellsOnDeleteKey = true; 
		/// <summary>
		/// When the user presses the DELETE key all selected ranges cells will be cleared
		/// </summary>
		public bool ClearCellsOnDeleteKey {
			get { return clearCellsOnDeleteKey; }
			set { clearCellsOnDeleteKey = value; }
		}

		//		private bool autoWrapNavigation;
		//		/// <summary>
		//		/// Determines if moving right or left wraps around to the first or last cell or stays in the current cell
		//		/// </summary>
		//		public bool AutoWrapNavigation
		//		{
		//			get { return autoWrapNavigation; }
		//			set 
		//			{ 
		//				autoWrapNavigation = value;
		//				CreateCurrentCellWorkaround();
		//			}
		//		}

		private bool autoHighlightRow = false;
		/// <summary>
		/// Determines if the the entire row is auto highlighted as the user navigates up and down the grid
		/// </summary>
		public bool AutoHighlightRow {
			get { return autoHighlightRow; }
			set {
				autoHighlightRow = value;
				if (value) {
					this.grid.TableControlPrepareViewStyleInfo += new Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlPrepareViewStyleInfoEventHandler(this.grid_TableControlPrepareViewStyleInfo);
				}
				else {
					this.grid.TableControlPrepareViewStyleInfo -= new Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlPrepareViewStyleInfoEventHandler(this.grid_TableControlPrepareViewStyleInfo);
				}
			}
		}
						
		private Hashtable toolTipValues = null;
		
		[Browsable(false)]
		public Hashtable ToolTipValues {
			get { return this.toolTipValues; }
			set { this.toolTipValues = value; }
		}

		private GridColumn toolTipDisplayColumn = null;
		/// <summary>
		/// Used in conjuction with ToolTipValues to detemrine the one-and-only (ATM)
		/// column which should have toltip displayed
		/// </summary>
		[Browsable(false)]
		public GridColumn ToolTipDisplayColumn {
			get { return toolTipDisplayColumn; }
			set { toolTipDisplayColumn = value; }
		}

		/// <summary>
		/// Show the Access-like Navigation bar
		/// </summary>
		public bool ShowNavigationBar {
			get { return grid.ShowNavigationBar; }
			set { grid.ShowNavigationBar = value; }
		}

		public System.Windows.Forms.BorderStyle BorderStyle {
			get { return this.grid.BorderStyle; }
			set { this.grid.BorderStyle = value; }
		}

		/// <summary>
		/// Determines the listbox selection mode
		/// </summary>
		public SelectionMode SelectionMode {
			get { return grid.TableOptions.ListBoxSelectionMode; }
			set { grid.TableOptions.ListBoxSelectionMode = value; }
		}

		/// <summary>
		/// Indicates whether the user can drag column headers and rearrange the order of columns.  
		/// </summary>
		public bool AllowDragColumns {
			get { return grid.TableOptions.AllowDragColumns; }
			set { grid.TableOptions.AllowDragColumns = value; }
		}
		/// <summary>
		/// Gets or sets the current display field
		/// </summary>
		[Browsable(false)]
		public FieldDescriptor CurrentField {
			get { return grid.Table.CurrentRecordManager.CurrentField; }
			set { grid.Table.CurrentRecordManager.CurrentField = value; }
		}

		/// <summary>
		/// Gets or sets the current record
		/// </summary>
		[Browsable(false)]
		public Record CurrentRecord {
			get { return grid.Table.CurrentRecord;}
			set { grid.Table.CurrentRecord = value;}
		}


		/// <summary>
		/// Provides a flat selection of visible elements in table
		/// </summary>
		[Browsable(false)]
		public DisplayElementsInTableCollection DisplayElements {
			get { return grid.Table.DisplayElements; }
		}

		//		/// <summary>
		//		/// Returns the visible columns in the grid
		//		/// </summary>
		//		[Browsable(false)]
		//		public GridVisibleColumnDescriptorCollection VisibleColumnss
		//		{
		//			get { return grid.TableDescriptor.VisibleColumns; }
		//		}

		/// <summary>
		/// Get or sets the collection of merged columns - used in conjucntion with InitQueryCoveredRangeEvent
		/// </summary>
		[Browsable(false)]
		public MergedColumnHeaderCollection MergedColumnHeaders {
			get { return mergedColumnHeaderCollection ; }
			set	{ this.mergedColumnHeaderCollection = value; }
		}

		/// <summary>
		/// Gets/sets the height of column cations. 32 is ideal for two lines
		/// </summary>
		public int ColumnHeaderHeight {
			get { return  grid.TableOptions.ColumnHeaderRowHeight; }
			set { grid.TableOptions.ColumnHeaderRowHeight = value; }
		}

		/// <summary>
		/// Managed Undo and Redo in grid
		/// </summary>
		public bool CommandStackEnabled {
			get { return grid.TableModel.CommandStack.Enabled;}
			set { grid.TableModel.CommandStack.Enabled = value; }
		}

		/// <summary>
		/// The GridTableModel is derived from GridModel and add support for retrieving data from the datasource
		/// </summary>	
		[Browsable(false)]
		public GridTableModel GridTableModel {
			get { return grid.TableModel; }
		}

		/// <summary>
		/// The Table object manages the records from the engines DataSource and provides access to records and grouped elements through several collection classes, most prominent the Syncfusion.Grouping.DisplayElementsInTableCollection.  
		/// </summary>
		[Browsable(false)]
		public GridTable GridTable {
			get { return grid.Table;}
		}

		/// <summary>
		/// Gets the hosted Syncfusion.Windows.Forms.Grid.Grouping.GridTableControl. The GridTableControl is a grid derived from GridControlBase and displays and allows user interaction and modification of data. 
		/// </summary>
		[Browsable(false)]
		public Syncfusion.Windows.Forms.Grid.GridControlBase GridControlBase {
			get	{return gridControlBase;}
		}

		[Browsable(false)]
		public GridGroupingControl SyncfusionGridControl {
			get { return grid; }
		}

		private bool showFullGroupingText = false;
		/// <summary>
		/// When true - grouping text showns the category name and value
		/// When false - grouping text showns just the category value
		/// </summary>
		public bool ShowFullGroupingText {
			get { return showFullGroupingText; }
			set {
				const string FULL_CAPTION = "{CategoryName}: {Category}";
				const string SHORT_CAPTION = "{Category}";

				if (value) {
					grid.TableDescriptor.ChildGroupOptions.CaptionText = FULL_CAPTION;
				}
				else {
					grid.TableDescriptor.ChildGroupOptions.CaptionText = SHORT_CAPTION;
				}
			}
		}

		public bool ShowRowHeader {
			get { return grid.TableOptions.ShowRowHeader;}
			set { grid.TableOptions.ShowRowHeader = value;}
		}

		public override Font Font {
			get { return grid.Font; }
			set { grid.Font = value; }
		}

		private bool useAlphaBlendHighLighting = false;
		/// <summary>
		/// Use AlphaBlendng to highlight selected cells - invert background
		/// Currently allows ANY cell to be highlighted
		/// </summary>
		public bool UseAlphaBlendHighLighting {
			get{ return useAlphaBlendHighLighting;}
			set {
				useAlphaBlendHighLighting = value;

				if (value) {
					grid.TableOptions.AllowSelection = GridSelectionFlags.AlphaBlend | GridSelectionFlags.Cell;
				}
				else {
					//this.gridGroupingControl3.TableOptions.AllowSelection = ((Syncfusion.Windows.Forms.Grid.GridSelectionFlags)((Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Cell | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.AlphaBlend)));
					grid.TableOptions.AllowSelection = GridSelectionFlags.Cell;
				}

				
			}
		}

		public Color GridBackColor {
			get { return grid.TableControl.BackColor;  }
			set { grid.TableControl.BackColor = value; }
		}

		public bool ShowGroupDropArea {
			get {
				return this.grid.ShowGroupDropArea;
			}
			set {
				this.grid.ShowGroupDropArea = value;
			}
		}

		#endregion

		public GridRecord Find(string text) {
			GridRecord rec = null;
			GridRecord oldRec = (GridRecord) this.grid.Table.CurrentRecord;
			this.grid.BeginUpdate();
			try {
				string colName="";
				if (grid.TableDescriptor.GroupedColumns.Count > 0) {
					colName = this.grid.TableDescriptor.GroupedColumns[0].Name;
				} 
				else if (this.grid.TableDescriptor.SortedColumns.Count > 0) {	
					colName = this.grid.TableDescriptor.SortedColumns[0].Name;
				}
				else {
					return rec;
				}
				
				RecordFilterDescriptor filter = new RecordFilterDescriptor("quickSearch");
				filter.Expression = "[" + colName + "] LIKE '" + text + "*'";
				this.grid.TableDescriptor.RecordFilters.Add(filter);

				if (this.grid.Table.FilteredRecords.Count > 0) {
					rec = (GridRecord) this.grid.Table.FilteredRecords[0];
				}
				this.grid.TableDescriptor.RecordFilters.Clear();
			}
			catch (Exception ex) {
				
				System.Console.WriteLine(ex.Message + " " + ex.StackTrace );
				this.grid.TableDescriptor.RecordFilters.Clear();
			} 
			
			
			if (rec!=null) {
				this.grid.Table.CurrentRecord = rec;
			} 
			else {
				if (oldRec != null ) {
					this.grid.Table.CurrentRecord = oldRec;
				}
			}
			this.grid.EndUpdate(true);
			return rec;
		}
	

		
		#region Find Next Commented
		/* Find Next 
		   public int FindNext(string text, bool moveUp) {
			int recPos = -1;
			GridRecord rec = null;
			GridRecord oldRec = (GridRecord) this.grid.Table.CurrentRecord;
			int currentRecPos = this.grid.Table.Records.IndexOf(oldRec);
			this.grid.BeginUpdate();
			try {
				string colName="";
				if (grid.TableDescriptor.GroupedColumns.Count > 0) {
					colName = this.grid.TableDescriptor.GroupedColumns[0].Name;
				} 
				else if (this.grid.TableDescriptor.SortedColumns.Count > 0) {	
					colName = this.grid.TableDescriptor.SortedColumns[0].Name;
				}
				else {
					return recPos;
				}
				
				RecordFilterDescriptor filter = new RecordFilterDescriptor("quickSearch");
				filter.Expression = "[" + colName + "] LIKE '*" + text + "*'";
				this.grid.TableDescriptor.RecordFilters.Add(filter);

				if (this.grid.Table.FilteredRecords.Count > 0) {
					if ( !moveUp) {
						foreach(GridRecord filterRec in this.grid.Table.FilteredRecords) {
							int filterRecPos = this.grid.Table.Records.IndexOf(filterRec);
							if (filterRecPos > currentRecPos) {
								rec = filterRec;
								break;
							}
						}
					} else {
						GridRecord oldFilterRec = null;
						int oldFilterRecPos = 0;
						foreach(GridRecord filterRec in this.grid.Table.FilteredRecords) {
							int filterRecPos = this.grid.Table.Records.IndexOf(filterRec);
							if (filterRecPos >= currentRecPos) {
								break;
							}
							oldFilterRec = filterRec;
							oldFilterRecPos = filterRecPos;
						}
						rec = oldFilterRec;
					}
				}

				this.grid.TableDescriptor.RecordFilters.Clear();
			}
			catch (Exception ex) {
				
				System.Console.WriteLine(ex.Message + " " + ex.StackTrace );
				this.grid.TableDescriptor.RecordFilters.Clear();
				return recPos;
			} 
			
			
			if (rec!=null) {
				this.grid.Table.CurrentRecord = rec;
				recPos = this.grid.Table.Records.IndexOf(rec);
				this.grid.TableControl.TopRowIndex = recPos;
				this.grid.TableControl.UpdateScrollBars();
			} 
			else {
				if (oldRec != null ) {
					this.grid.Table.CurrentRecord = oldRec;
				}
			}
			
			this.grid.EndUpdate(true);
			return recPos;
		}*/
		#endregion

		#region Events
		/// <summary>
		/// Allows consumer to hook into the cell value chaging
		/// </summary>
		public event Syncfusion.Grouping.RecordValueChangingEventHandler RecordValueChanging;

		/// <summary>
		/// Allows consumer to hook into the cell value chaging
		/// </summary>
		public event Syncfusion.Grouping.RecordValueChangedEventHandler RecordValueChanged;

		/// <summary>
		/// Allows consumer to hook into activating a new cell 
		/// </summary>
		public event GridTableControlEventHandler CurrentCellActivated;	

		/// <summary>
		/// allows consumer to be notifed when a user hovers over a record cell rready for  tool tip
		/// </summary>
		public event GridTableControlPrepareViewStyleInfoEventHandler QueryRecordCellToolTip;

		/// <summary>
		/// Allows a consumer to show a custom header col tool tip
		/// </summary>
		public event GridTableControlPrepareViewStyleInfoEventHandler QueryColumnHeaderToolTip;

		public event Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlCellClickEventHandler CellDoubleClicked;
		public event GridQueryCustomSummaryEventHandler QueryCustomSummary;
		#endregion
	
		#region Grid Styles Properties
		/// <summary>
		/// Sets the style of grouping related objects - note the syncfusion grid
		/// does not uses references - so setting one style property is not copied to
		/// others that may have bene referencing the style - or so they thought
		/// </summary>
		public GridStyleFormatter GroupingStyle {
			get { return groupingStyle; }
		}

		/// <summary>
		/// Sets the style of headers 
		/// </summary>
		public GridStyleFormatter HeaderStyle {
			get { return headerStyle; }
		}

		/// <summary>
		/// Sets the style of datarows 
		/// </summary>
		public GridStyleFormatter DataRowStyle {
			get { return dataRowStyle; }
		}

		#endregion

		#region Private Style methods

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() {
			Syncfusion.Windows.Forms.Grid.Grouping.GridTableBaseStyle gridTableBaseStyle1 = new Syncfusion.Windows.Forms.Grid.Grouping.GridTableBaseStyle("test");
			this.grid = new Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl();
			this.groupingConfig1 = new Syncfusion.Grouping.GroupingConfig();
			this.screenShot = new System.Windows.Forms.PictureBox();
			((System.ComponentModel.ISupportInitialize)(this.grid)).BeginInit();
			this.SuspendLayout();
			// 
			// grid
			// 
			this.grid.Appearance.AnyGroupCell.Enabled = false;
			this.grid.Appearance.AnyHeaderCell.Enabled = false;
			this.grid.Appearance.TopLeftHeaderCell.AllowEnter = false;
			this.grid.Appearance.TopLeftHeaderCell.Interior = new Syncfusion.Drawing.BrushInfo(System.Drawing.SystemColors.Window);
			this.grid.BackColor = System.Drawing.SystemColors.Control;
			gridTableBaseStyle1.Name = "test";
			gridTableBaseStyle1.StyleInfo.AllowEnter = true;
			this.grid.BaseStyles.AddRange(new Syncfusion.Windows.Forms.Grid.Grouping.GridTableBaseStyle[] {
																											  gridTableBaseStyle1});
			this.grid.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.grid.CausesValidation = false;
			this.grid.ChildGroupOptions.CaptionText = "{Category} ";
			this.grid.Dock = System.Windows.Forms.DockStyle.Fill;
			this.grid.Font = new System.Drawing.Font("Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.grid.Location = new System.Drawing.Point(0, 0);
			this.grid.Name = "grid";
			this.grid.Size = new System.Drawing.Size(592, 176);
			this.grid.TabIndex = 12;
			this.grid.TableDescriptor.AllowNew = false;
			this.grid.TableDescriptor.AllowRemove = false;
			this.grid.TableOptions.ShowRowHeader = true;
			this.grid.TableOptions.ShowTreeLines = true;
			this.grid.Text = "gridGroupingControl";
			this.grid.TopLevelGroupOptions.ShowCaption = false;
			this.grid.QueryCustomSummary += new Syncfusion.Windows.Forms.Grid.Grouping.GridQueryCustomSummaryEventHandler(this.grid_QueryCustomSummary);
			this.grid.TableControlCellDoubleClick += new Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlCellClickEventHandler(this.grid_TableControlCellDoubleClicked);
			this.grid.RecordValueChanged += new Syncfusion.Grouping.RecordValueChangedEventHandler(this.grid_RecordValueChanged);
			this.grid.TableControlQueryAllowSortColumn += new Syncfusion.Windows.Forms.Grid.Grouping.GridQueryAllowSortColumnEventHandler(this.grid_TableControlQueryAllowSortColumn);
			this.grid.TableControlCurrentCellActivated += new Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlEventHandler(this.grid_TableControlCurrentCellActivated);
			this.grid.RecordValueChanging += new Syncfusion.Grouping.RecordValueChangingEventHandler(this.grid_RecordValueChanging);
			// 
			// screenShot
			// 
			this.screenShot.Dock = System.Windows.Forms.DockStyle.Fill;
			this.screenShot.Location = new System.Drawing.Point(0, 0);
			this.screenShot.Name = "screenShot";
			this.screenShot.Size = new System.Drawing.Size(592, 176);
			this.screenShot.TabIndex = 13;
			this.screenShot.TabStop = false;
			// 
			// GroupingGrid
			// 
			this.CausesValidation = false;
			this.Controls.Add(this.grid);
			this.Controls.Add(this.screenShot);
			this.Name = "GroupingGrid";
			this.Size = new System.Drawing.Size(592, 176);
			((System.ComponentModel.ISupportInitialize)(this.grid)).EndInit();
			this.ResumeLayout(false);

		}		

		#endregion

		#region Custom Event Handlers

		private void grid_QueryCustomSummary(object sender, Syncfusion.Windows.Forms.Grid.Grouping.GridQueryCustomSummaryEventArgs e) {
			if (QueryCustomSummary != null) {
				QueryCustomSummary(sender, e);
			}
		}

		//3.0.1.0 change
		private void grid_RecordValueChanging(object sender, Syncfusion.Grouping.RecordValueChangingEventArgs e) {
			try {
				// If the user has hooked into this event rasie it up to them
				if (RecordValueChanging != null) {
					RecordValueChanging(sender, e);
				}
			}
			catch(Exception ex) {
				//Logger.WriteLogEntry("RecordValueChanging", ex, LogType.Error);
			}
		
		}

		private void grid_RecordValueChanged(object sender, Syncfusion.Grouping.RecordValueChangedEventArgs e) {
			try {
				// If the user has hooked into this event rasie it up to them
				if (RecordValueChanged != null) {
					RecordValueChanged(sender, e);
				}
			}
			catch(Exception ex) {
				//Logger.WriteLogEntry("RecordValueChanged", ex, LogType.Error);
			}
		
		}
		
		/// <summary>
		/// With the grouping gird to have to intercept QueryCoveredRange and handle it all yourself!!!
		/// </summary>
		public void TableModel_QueryCoveredRange(object sender, Syncfusion.Windows.Forms.Grid.GridQueryCoveredRangeEventArgs e) {
			try {
				GridTableModel tm = sender as GridTableModel;
				Element el = null;

				// Put in some addtional error hadnlign if there's no rows shown - filtering could cuase that
				if (e.RowIndex <= tm.Table.DisplayElements.Count -1) {
					el = tm.Table.DisplayElements[e.RowIndex];
				}

				if (el != null
					&&
					el is ColumnHeaderRow) {
					GridTableCellStyleInfo style = tm[e.RowIndex, e.ColIndex];

					// There's some funny stuff going on with the gird - sometimes it
					// gets out of sync with data there and the column becomes null
					if (style.TableCellIdentity.Column != null) {
						// We now have to go through our merged col collection looking for
						// for this column - this could break if they're not next to each other!!!
						MergedColumnHeader mergedColumnHeader = 
							mergedColumnHeaderCollection.FindWithMappingName(style.TableCellIdentity.Column.MappingName);

						if (mergedColumnHeader != null) {
							int startCol = el.GroupLevel+1;
							int colIndex = e.ColIndex;

							// If it's the seocnd col decrement the col index
							if (style.TableCellIdentity.Column.MappingName == mergedColumnHeader.GridColumn2.MappingName) {
								colIndex--;
							}

							e.Range = GridRangeInfo.Cells(e.RowIndex, colIndex, e.RowIndex, colIndex + 1);
							e.Handled = true;
						}
					}

				}
			}

			catch (Exception ex) {
				// Not sure why but log it
				//Logger.WriteLogEntry(ex.ToString());
			}

		}

		private void TableModel_SelectionChanged(object sender, Syncfusion.Windows.Forms.Grid.GridSelectionChangedEventArgs e) {
			GridTableModel TableModel = (GridTableModel) sender;
			if(TableModel.CurrentCellInfo != null) {
				int row =-2, col=-2;
				grid.TableControl.PointToRowCol(grid.PointToClient(Control.MousePosition), out row, out col);
				if( grid.TableControl.GetCellRenderer(row, col) is GridCheckBoxCellRendererEx) {
					if(e.Reason == GridSelectionReason.MouseDown || e.Reason == GridSelectionReason.MouseMove) {
						if(row == TableModel.CurrentCellInfo.RowIndex && col == TableModel.CurrentCellInfo.ColIndex) {
							TableModel.Model[TableModel.CurrentCellInfo.RowIndex, TableModel.CurrentCellInfo.ColIndex].CellValue = !(bool)TableModel.Model[TableModel.CurrentCellInfo.RowIndex, TableModel.CurrentCellInfo.ColIndex].CellValue;
						}
						else {
							TableModel.Model[row, col].CellValue = TableModel.Model[TableModel.CurrentCellInfo.RowIndex, TableModel.CurrentCellInfo.ColIndex].CellValue;
						}
					}			
				}
			}            
		}

		private void gridGroupingControl1_TableControlMouseDown(object sender, GridTableControlMouseEventArgs e) {
			GridGroupingControl GGC = (GridGroupingControl) sender;
			GridTableModel TableModel = GGC.TableModel;
			int row=-2, col=-2;
			grid.TableControl.PointToRowCol(grid.PointToClient(Control.MousePosition), out row, out col);
			grid.TableControl.CurrentCell.MoveTo(row, col);            
		}

		private DataRowView currentDataRowView = null;
		private void cm_PositionChanged(object sender, EventArgs e) {
			// The first time this is set - using singleSec you might get the following error
			//A first chance exception of type 'System.IndexOutOfRangeException' occurred in system.windows.forms.dll
			//Additional information: No value at index -1.
			// so just wrap it in a try and hide it
			try {
				this.currentDataRowView = this.currencyManager.Current as DataRowView;
			}
			catch{}
			
		}

	
		private void grid_TableControlPrepareViewStyleInfo(object sender, GridTableControlPrepareViewStyleInfoEventArgs e) {
			Element el = e.TableControl.Table.DisplayElements[e.Inner.RowIndex];

			if (QueryColumnHeaderToolTip!= null && el is GridColumnHeaderRow) {
				QueryColumnHeaderToolTip(sender, e);
			}
			else if (this.currentDataRowView != null 
				&& 
				el is GridRecordRow) {
				GridRecordRow rec = el as GridRecordRow;
				if(rec != null) {
					DataRowView drv = rec.GetData() as DataRowView;
					GridCurrentCell currentCell = gridControlBase.CurrentCell;

					// Work out if we're query the stlye for the current cell
					bool isCurrentCellStyle = false;
					if (currentCell != null
						&&
						currentCell.RowIndex == e.Inner.RowIndex
						&&
						currentCell.ColIndex == e.Inner.ColIndex) {
						isCurrentCellStyle = true;
					}
 
					if (drv != null 
						&& 
						drv.Equals(this.currentDataRowView)
						&&
						! isCurrentCellStyle) {
						e.Inner.Style.Interior = highlightRowBrush;    
						e.Inner.Style.TextColor = SystemColors.HighlightText;          
					}
				}
				
				
				if (toolTipValues != null && toolTipDisplayColumn != null) {
					SetToolTip(e);
				}
				else {
					if (QueryRecordCellToolTip != null) {
						QueryRecordCellToolTip(sender, e);			
					}
				}

			}
		}

		private bool ignoreCurrentCellActivatedEvent;

	
		public bool IgnoreCurrentCellActivatedEvent {
			get { return ignoreCurrentCellActivatedEvent; }
			set { ignoreCurrentCellActivatedEvent = value; }
		}

		private void grid_TableControlCurrentCellActivated(object sender, Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlEventArgs e) {
			try {
				// Fire the event if hooked into AND the client wants it fired
				if (CurrentCellActivated != null && ! ignoreCurrentCellActivatedEvent) {
					CurrentCellActivated(sender, e);
				}
			}
			catch(Exception ex) {
				//Logger.WriteLogEntry("TableControlCurrentCellActivated", ex, LogType.Error);
			}

		}

		private void grid_TableControlCellDoubleClicked(object sender, Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlCellClickEventArgs e) {
			try {
				Element item = this.grid.TableModel.GetDisplayElementAt(e.Inner.RowIndex);
				if (item is GridRecordRow) {
					GridTableCellStyleInfo style = grid.TableModel[e.Inner.RowIndex, e.Inner.ColIndex];
					if (CellDoubleClicked !=null) {
						if (style.TableCellIdentity.Column != null) {
							CellDoubleClicked(sender, e);
						} 
						else if (e.Inner.ColIndex == 0) {
							CellDoubleClicked(sender, e);
						}
					}
				}
			} 
			catch (Exception ex) {
				//Logger.WriteLogEntry("TableControlCellDoubleClicked", ex, LogType.Error);
			}
		}

		//		// Thorugh this event we can delete the contents of the cell if they press
		//		// the delete key
		//		private void grid_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e) 
		//		{
		//			switch (e.KeyCode) 
		//			{
		//				case Keys.Delete:
		//					if (clearCellsOnDeleteKey
		//						&&
		//						grid.TableModel.Selections.Ranges.ActiveRange != null) 
		//					{
		//						e.Handled = true;
		//						grid.TableModel.CutPaste.Cut();
		//					}
		//					break;
		//
		//			}
		//		
		//		}
		#endregion

		#region Private Stuff

	
		private void SetToolTip(GridTableControlPrepareViewStyleInfoEventArgs e) {

			try {
				GridTableCellStyleInfo style = grid.TableModel[e.Inner.RowIndex, e.Inner.ColIndex];
				if (style.TableCellIdentity.Column != null
					&&
					style.TableCellIdentity.Column.MappingName == toolTipDisplayColumn.MappingName) {
					GridRecordRow recordRow = 
						(GridRecordRow)grid.TableModel.GetDisplayElementAt(e.Inner.RowIndex);
					Record record = (Record)recordRow.ParentRecord;
					DataRowView rowView = (DataRowView)record.GetData();

					object key = GetPrimaryKey(record);
					if (key != null 
						&&
						toolTipValues[key] != null) {
						e.Inner.Style.CellTipText = toolTipValues[key].ToString();
					}

				}
			}
			catch (Exception ex) {
				////Logger.WriteLogEntry("Exception in SetToolTip:" + ex.Message);
			}

		}

		/// <summary>
		/// Debguggin info
		/// </summary>
		/// <returns></returns>
		public string CurrentCellInfo() {

			string retVal = "";

			GridCurrentCell currentCell = gridControlBase.CurrentCell;
			if (currentCell == null) {
				retVal = "No CurrentCell";
			}
			else {
				retVal = " C=" + currentCell.ColIndex + " R=" + currentCell.RowIndex +
					" isActive=" + currentCell.IsActive + " Info=" + currentCell.Info;
			}
			
			return retVal;
		}

		private void grid_TableControlQueryAllowSortColumn(object sender, Syncfusion.Windows.Forms.Grid.Grouping.GridQueryAllowSortColumnEventArgs e) {
			// Note there's a bug in the grid.  If a user enters a new position
			// presses enter and thne clciks on a header col to sort
			// the data, the grid does resort but keeps the original
			// currentcell in exactly the same row/col postion
			grid.TableControl.CurrentCell.Deactivate(false);
			//grid.TableControl.CurrentCell.EndEdit();
			
		}
		
		#endregion

		#region Debugging code commented out - but kept as handy
		//		// These have been left in for debugging performance
		//		private void grid_SourceListRecordChanging(object sender, Syncfusion.Grouping.RecordChangedEventArgs e)
		//		{
		//		//Logger.WriteLogEntry("grid_SourceListRecordChanging");
		//		}
		//
		//		private void grid_InvalidatingSummaries(object sender, Syncfusion.Grouping.TableEventArgs e)
		//		{
		//		//Logger.WriteLogEntry("grid_InvalidatingSummaries");
		//		}
		//

		//
		//		private void ListPropertyChanged(object sender, Syncfusion.Collections.ListPropertyChangedEventArgs e)
		//		{
		//			////Logger.WriteLogEntry("ListPropertyChangedEventHandler");
		//		}
		//
		//		private void grid_TableControl_Paint(object sender, PaintEventArgs e)
		//		{
		//			// Write to log to see know it's not painting too much
		//			//Logger.WriteLogEntry("grid.paint - too much is bad for performance");
		//		}

		//		private void grid_GroupSummaryInvalidated(object sender, Syncfusion.Grouping.GroupEventArgs e)
		//		{
		//			//Logger.WriteLogEntry("grid.GroupSummaryInvalidated - too much is bad for performance");
		//		}
		//
		//		private void grid_InvalidatingSummaries(object sender, Syncfusion.Grouping.TableEventArgs e)
		//		{
		//			//Logger.WriteLogEntry("grid.InvalidatingSummaries - too much is bad for performance");
		//		}
		#endregion
	
#endregion
	}
}
